@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2002, 2005, 2007, 2009-2012, 2014-2016 Free
+   Copyright (C) 1999-2002, 2005, 2007, 2009-2012, 2014-2017 Free
    Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -144,7 +144,7 @@ expand_line (const char *str, mu_message_t msg)
 
   if (!*str)
     return NULL;
-  mu_opool_create (&pool, 1);
+  mu_opool_create (&pool, MU_OPOOL_ENOMEMABRT);
   for (p = str; *p; p++)
     {
       switch (*p)
