@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2004, 2007, 2010 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999-2001, 2004, 2007, 2010-2012, 2014-2017 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* This is an example program to illustrate the use of stream functions.
    It connects to a remote HTTP server and prints the contents of its
@@ -51,19 +49,20 @@ http_stream_wait (mu_stream_t stream, int flags, size_t *attempt)
       tv.tv_sec = io_timeout;
       tv.tv_usec = 0;
       rc = mu_stream_wait (stream, &oflags, &tv);
-      switch (rc) {
-      case 0:
-	if (flags & oflags)
-	  return 0;
-	/* FALLTHROUGH */
-      case EAGAIN:
-      case EINPROGRESS:
-	++*attempt;
-	continue;
-	
-      default:
-	return rc;
-      }
+      switch (rc)
+	{
+	case 0:
+	  if (flags & oflags)
+	    return 0;
+	  /* FALLTHROUGH */
+	case EAGAIN:
+	case EINPROGRESS:
+	  ++*attempt;
+	  continue;
+	  
+	default:
+	  return rc;
+	}
     }
   return ETIMEDOUT;
 }
@@ -71,9 +70,9 @@ http_stream_wait (mu_stream_t stream, int flags, size_t *attempt)
 int
 main (int argc, char **argv)
 {
-  int ret, off = 0;
+  int ret;
   mu_stream_t stream;
-  size_t nb, size;
+  size_t nb;
   size_t attempt;
   char *url = "www.gnu.org";
 
@@ -90,37 +89,32 @@ main (int argc, char **argv)
 	    argc == 3 ? argv[2] : "/");
 
   ret = mu_tcp_stream_create (&stream, url, 80, MU_STREAM_NONBLOCK);
-  if (ret != 0)
+  if (ret != 0 && !(ret == EAGAIN || ret == EINPROGRESS))
     {
       mu_error ("mu_tcp_stream_create: %s", mu_strerror (ret));
       exit (EXIT_FAILURE);
     }
 
-  for (attempt = 0; (ret = mu_stream_open (stream)); )
+  for (attempt = 0; ret; )
     {
       if ((ret == EAGAIN || ret == EINPROGRESS) && attempt < io_attempts)
 	{
 	  ret = http_stream_wait(stream, MU_STREAM_READY_WR, &attempt);
 	  if (ret == 0)
-	    continue;
+	    {
+	      ret = mu_stream_open (stream);
+	      continue;
+	    }
 	}
       mu_error ("mu_stream_open: %s", mu_strerror (ret));
       exit (EXIT_FAILURE);
     }
 
-  for (attempt = 0, size = strlen (wbuf); size > 0; )
+  for (attempt = 0;; )
     {
-      ret = mu_stream_write (stream, wbuf + off, strlen (wbuf), 0, &nb);
+      ret = mu_stream_write (stream, wbuf, strlen (wbuf), NULL);
       if (ret == 0)
-	{
-	  if (nb == 0)
-	    {
-	      mu_error("mu_stream_write: wrote 0 bytes");
-	      exit (EXIT_FAILURE);
-	    }
-	  off += nb;
-	  size -= nb;
-	}
+	break;
       else if (ret == EAGAIN)
         {
 	  if (attempt < io_attempts)
@@ -149,7 +143,7 @@ main (int argc, char **argv)
   attempt = 0;
   for (;;)
     {
-      ret = mu_stream_read (stream, rbuf, sizeof (rbuf), 0, &nb);
+      ret = mu_stream_read (stream, rbuf, sizeof (rbuf), &nb);
       if (ret == 0)
 	{
 	  if (nb == 0)
@@ -182,6 +176,6 @@ main (int argc, char **argv)
       exit (EXIT_FAILURE);
     }
 
-  mu_stream_destroy (&stream, NULL);
+  mu_stream_destroy (&stream);
   exit (EXIT_SUCCESS);
 }
