@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2007-2012, 2014-2016 Free Software
+   Copyright (C) 1999, 2001, 2007-2012, 2014-2017 Free Software
    Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -49,7 +49,7 @@ imap4d_uid (struct imap4d_session *session,
     rc = imap4d_search0 (tok, 1, &err_text);
   else
     {
-      err_text = "Uknown uid command";
+      err_text = "Unknown uid command";
       rc = RESP_BAD;
     }
   return io_completion_response (command, rc, "%s %s", cmd, err_text);
