@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2007, 2010, 2011 Free Software Foundation, Inc.
+   Copyright (C) 2007, 2010-2012, 2014-2017 Free Software Foundation,
+   Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -63,4 +64,8 @@ int mu_acl_check_fd (mu_acl_t acl, int fd, mu_acl_result_t *pres);
 int mu_acl_action_to_string (mu_acl_action_t act, const char **pstr);
 int mu_acl_string_to_action (const char *str, mu_acl_action_t *pres);
 
+int mu_acl_setenv (mu_acl_t acl, const char *name, const char *val);
+const char *mu_acl_getenv (mu_acl_t acl, const char *name);
+int mu_acl_set_session_id (mu_acl_t acl);
+
 #endif
