@@ -1,7 +1,6 @@
-/*
-   GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2004, 2006, 2007, 2009, 2010 Free Software Foundation,
-   Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 2004, 2006-2007, 2009-2012, 2014-2017 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -14,10 +13,8 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA
-*/
+   Public License along with this library.  If not, see
+   <http://www.gnu.org/licenses/>. */
 
 #ifndef _MUCPP_LIST_H
 #define _MUCPP_LIST_H
@@ -28,7 +25,6 @@
 #include <mailutils/cpp/error.h>
 #include <mailutils/cpp/iterator.h>
 
-typedef int mu_list_action_t (void*, void*);
 typedef int (*mu_list_comparator_t) (const void*, const void*);
 
 namespace mailutils
@@ -67,7 +63,7 @@ class List
   void to_array (void** array, size_t count, size_t* pcount);
   void locate (void* item, void** ret_item);
 
-  void apply (mu_list_action_t* action, void* cbdata);
+  void apply (mu_list_action_t action, void* cbdata);
   mu_list_comparator_t set_comparator (mu_list_comparator_t comp);
   mu_list_destroy_item_t set_destroy_item (mu_list_destroy_item_t mu_destroy_item);
 
