@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2004, 2007, 2008, 2009,
-   2010 Free Software Foundation, Inc.
+   Copyright (C) 1999-2004, 2007-2012, 2014-2017 Free Software
+   Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -12,10 +12,10 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
 
-   You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA */
+   You should have received a copy of the GNU General Public License
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
+
+#include <mailutils/types.h>
 
 #include <mailutils/acl.h>
 #include <mailutils/address.h>
@@ -25,12 +25,14 @@
 #include <mailutils/attribute.h>
 #include <mailutils/auth.h>
 #include <mailutils/body.h>
+#include <mailutils/datetime.h>
 #include <mailutils/debug.h>
 #include <mailutils/envelope.h>
 #include <mailutils/errno.h>
 #include <mailutils/error.h>
 #include <mailutils/filter.h>
 #include <mailutils/folder.h>
+#include <mailutils/glob.h>
 #include <mailutils/header.h>
 #include <mailutils/iterator.h>
 #include <mailutils/kwd.h>
@@ -44,7 +46,7 @@
 #include <mailutils/mime.h>
 #include <mailutils/monitor.h>
 #include <mailutils/mu_auth.h>
-#include <mailutils/mutil.h>
+#include <mailutils/util.h>
 #include <mailutils/nls.h>
 #include <mailutils/observer.h>
 #include <mailutils/opool.h>
@@ -56,14 +58,19 @@
 #include <mailutils/registrar.h>
 #include <mailutils/stream.h>
 #include <mailutils/tls.h>
-#include <mailutils/types.h>
 #include <mailutils/url.h>
-#include <mailutils/gocs.h>
 #include <mailutils/version.h>
-#include <mailutils/vartab.h>
 #include <mailutils/io.h>
 #include <mailutils/secret.h>
 #include <mailutils/cctype.h>
 #include <mailutils/cstr.h>
+#include <mailutils/wordsplit.h>
+#include <mailutils/log.h>
+#include <mailutils/syslog.h>
+#include <mailutils/stdstream.h>
+#include <mailutils/prog.h>
+#include <mailutils/sockaddr.h>
+#include <mailutils/cidr.h>
+#include <mailutils/msgset.h>
 
 /* EOF */
