@@ -1,6 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010,
-   2011 Free Software Foundation, Inc.
+   Copyright (C) 2002-2012, 2014-2017 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -51,12 +50,112 @@
 #include <mailutils/sql.h>
 #include <mailutils/cstr.h>
 #include <mailutils/wordsplit.h>
+#include <mailutils/cli.h>
+#include <mailutils/kwd.h>
 #include "sql.h"
 
 #ifdef USE_SQL
 
-struct mu_internal_sql_config mu_sql_module_config;
+struct mu_sql_module_config mu_sql_module_config;
 
+/* Resource file configuration */
+
+static struct mu_kwd password_encryption[] = {
+  { "plain", mu_sql_password_plaintext },
+  { "scrambled", mu_sql_password_scrambled },
+  { "hash", mu_sql_password_hash },
+  { "crypt", mu_sql_password_hash },
+  { NULL }
+};
+
+static int
+cb_password_encryption (void *data, mu_config_value_t *val)
+{
+  int res;
+  
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
+    return 1;
+
+  if (mu_kwd_xlat_name (password_encryption, val->v.string, &res))
+    mu_error ("%s", _("unrecognized password encryption"));
+  else
+    mu_sql_module_config.password_encryption = res;
+  return 0;
+}
+
+static int
+cb_field_map (void *data, mu_config_value_t *val)
+{
+  char *err_term;
+  int rc = mu_cfg_field_map (val, &mu_sql_module_config.field_map, &err_term);
+
+  if (rc)
+    {
+      if (err_term)
+	mu_error (_("error near %s: %s"), err_term, mu_strerror (rc));
+      else
+	mu_error ("%s", mu_strerror (rc));
+    }
+
+  return rc;
+}
+
+static int
+cb_interface (void *data, mu_config_value_t *val)
+{
+  if (mu_cfg_assert_value_type (val, MU_CFG_STRING))
+    return 1;
+  mu_sql_module_config.interface = mu_sql_interface_index (val->v.string);
+  if (mu_sql_module_config.interface == 0)
+    {
+      mu_error (_("unknown SQL interface `%s'"), val->v.string);
+      return 1;
+    }
+  return 0;
+}
+  
+static struct mu_cfg_param mu_sql_param[] = {
+  { "interface", mu_cfg_callback, &mu_sql_module_config.interface, 0,
+    cb_interface,
+    N_("Set SQL interface to use."),
+    /* TRANSLATORS: Words to the right of : are keywords - do not translate */
+    N_("iface: mysql|odbc|postgres") },
+  { "getpwnam", mu_c_string, &mu_sql_module_config.getpwnam_query, 0, NULL,
+    N_("SQL query to use for getpwnam requests."),
+    N_("query") },
+  { "getpwuid", mu_c_string, &mu_sql_module_config.getpwuid_query, 0, NULL,
+    N_("SQL query to use for getpwuid requests."),
+    N_("query") },
+  { "getpass", mu_c_string, &mu_sql_module_config.getpass_query, 0, NULL,
+    N_("SQL query returning the user's password."),
+    N_("query") },
+  { "host", mu_c_string, &mu_sql_module_config.host, 0, NULL,
+    N_("SQL server host name.") },
+  { "user", mu_c_string, &mu_sql_module_config.user, 0, NULL,
+    N_("SQL user name.") },
+  { "passwd", mu_c_string, &mu_sql_module_config.passwd, 0, NULL,
+    N_("Password for the SQL user.") },
+  { "port", mu_c_int, &mu_sql_module_config.port, 0, NULL,
+    N_("SQL server port.") },
+  { "db", mu_c_string, &mu_sql_module_config.db, 0, NULL,
+    N_("Database name.") },
+  { "password-encryption", mu_cfg_callback, NULL, 0, cb_password_encryption,
+    N_("Type of password returned by getpass query."),
+    /* TRANSLATORS: Words to the right of : are keywords - do not translate */
+    N_("arg: plain|hash|crypt|scrambled") },
+  { "field-map", mu_cfg_callback, NULL, 0, cb_field_map,
+    N_("Set a field-map for parsing SQL replies.  The map is a "
+       "column-separated list of definitions.  Each definition has the "
+       "following form:\n"
+       "   <name: string>=<column: string>\n"
+       "where <name> is one of the following: name, passwd, uid, gid, "
+       "gecos, dir, shell, mailbox, quota, and <column> is the name of "
+       "the corresponding SQL column."),
+    N_("map: definition") },
+  { NULL }
+};
+
+
 static char *
 sql_escape_string (const char *ustr)
 {
@@ -88,97 +187,30 @@ sql_escape_string (const char *ustr)
 char *
 mu_sql_expand_query (const char *query, const char *ustr)
 {
+  int rc;
   char *res;
   char *esc_ustr;
-  struct mu_wordsplit ws;
-  const char *env[2 + 1];
 
   if (!query)
     return NULL;
 
   esc_ustr = sql_escape_string (ustr);
-  env[0] = "user";
-  env[1] = (char*) ustr;
-  env[2] = NULL;
-
-  ws.ws_env = env;
-  if (mu_wordsplit (query, &ws,
-		    MU_WRDSF_NOSPLIT | MU_WRDSF_NOCMD |
-		    MU_WRDSF_ENV | MU_WRDSF_ENV_KV))
-    {
-      mu_error (_("cannot expand line `%s': %s"), query,
-		mu_wordsplit_strerror (&ws));
-      return NULL;
-    }
-  else if (ws.ws_wordc == 0)
-    {
-      mu_error (_("expanding %s yields empty string"), query);
-      mu_wordsplit_free (&ws);
-      return NULL;
-    }
-  res = strdup (ws.ws_wordv[0]);
-  mu_wordsplit_free (&ws);
+  rc = mu_str_vexpand (&res, query, "user", esc_ustr, NULL);
   free (esc_ustr);
+  if (rc)
+    {
+      if (rc == MU_ERR_FAILURE)
+	{
+	  mu_error (_("cannot expand line `%s': %s"), query, res);
+	  free (res);
+	}
+      else
+	mu_error (_("cannot expand line `%s': %s"), query, mu_strerror (rc));
+      return NULL;
+    }
   return res;
 }
-
 
-static int
-decode_tuple_v1_0 (mu_sql_connection_t conn, int n,
-		   struct mu_auth_data **return_data)
-{
-  int rc;
-  char *mailbox_name = NULL;
-  char *name;
-      
-  if (mu_sql_get_column (conn, 0, 0, &name))
-    return MU_ERR_FAILURE;
-
-  if (n == 7)
-    {
-      char *tmp;
-      if (mu_sql_get_column (conn, 0, 6, &tmp))
-	return MU_ERR_FAILURE;
-      if (tmp && (mailbox_name = strdup (tmp)) == NULL)
-	return ENOMEM;
-    }
-  else if (mu_construct_user_mailbox_url (&mailbox_name, name))
-    return MU_ERR_FAILURE;
-      
-  if (mailbox_name)
-    {
-      char *passwd, *suid, *sgid, *dir, *shell;
-	  
-      if (mu_sql_get_column (conn, 0, 1, &passwd)
-	  || !passwd
-	  || mu_sql_get_column (conn, 0, 2, &suid)
-	  || !suid
-	  || mu_sql_get_column (conn, 0, 3, &sgid)
-	  || !sgid
-	  || mu_sql_get_column (conn, 0, 4, &dir)
-	  || !dir
-	  || mu_sql_get_column (conn, 0, 5, &shell)
-	  || !shell)
-	return MU_ERR_FAILURE;
-      
-      rc = mu_auth_data_alloc (return_data,
-			       name,
-			       passwd,
-			       atoi (suid),
-			       atoi (sgid),
-			       "SQL User",
-			       dir,
-			       shell,
-			       mailbox_name,
-			       1);
-    }
-  else
-    rc = MU_ERR_AUTH_FAILURE;
-  
-  free (mailbox_name);
-  return rc;
-}
-
 static int
 get_field (mu_sql_connection_t conn, const char *id, char **ret, int mandatory)
 {
@@ -206,8 +238,8 @@ get_field (mu_sql_connection_t conn, const char *id, char **ret, int mandatory)
 }
 
 static int
-decode_tuple_new (mu_sql_connection_t conn, int n,
-		  struct mu_auth_data **return_data)
+decode_tuple (mu_sql_connection_t conn, int n,
+	      struct mu_auth_data **return_data)
 {
   int rc;
   char *mailbox_name = NULL;
@@ -248,6 +280,8 @@ decode_tuple_new (mu_sql_connection_t conn, int n,
     {
     case 0:
       mailbox_name = strdup (mailbox_name);
+      if (!mailbox_name)
+        return ENOMEM;
       break;
       
     case MU_ERR_NOENT:
@@ -314,16 +348,6 @@ decode_tuple_new (mu_sql_connection_t conn, int n,
   return rc;
 }  
 
-static int
-decode_tuple (mu_sql_connection_t conn, int n,
-	      struct mu_auth_data **return_data)
-{
-  if (mu_sql_module_config.field_map || !mu_sql_module_config.positional)
-    return decode_tuple_new (conn, n, return_data);
-  else
-    return decode_tuple_v1_0 (conn, n, return_data);
-}
- 
 static int
 mu_auth_sql_by_name (struct mu_auth_data **return_data,
 		     const void *key,
@@ -626,7 +650,7 @@ mu_sql_authenticate (struct mu_auth_data **return_data MU_ARG_UNUSED,
 {
   const struct mu_auth_data *auth_data = key;
   char *pass = call_data;
-  char *sql_pass;
+  char *sql_pass, *crypt_pass;
   int rc;
   
   if (!auth_data)
@@ -635,13 +659,17 @@ mu_sql_authenticate (struct mu_auth_data **return_data MU_ARG_UNUSED,
   if ((rc = mu_sql_getpass (auth_data->name, &sql_pass)))
     return rc;
 
-  switch (mu_sql_module_config.password_type)
+  switch (mu_sql_module_config.password_encryption)
     {
-    case password_hash:
-      rc = strcmp (sql_pass, crypt (pass, sql_pass));
+    case mu_sql_password_hash:
+      crypt_pass = crypt (pass, sql_pass);
+      if (!crypt_pass)
+        rc = 1;
+      else
+        rc = strcmp (sql_pass, crypt_pass);
       break;
 
-    case password_scrambled:
+    case mu_sql_password_scrambled:
       /* FIXME: Should this call be implementation-independent? I mean,
          should we have mu_sql_check_scrambled() that will match the
 	 password depending on the exact type of the underlying database,
@@ -653,7 +681,7 @@ mu_sql_authenticate (struct mu_auth_data **return_data MU_ARG_UNUSED,
 #endif
       break;
 
-    case password_plaintext:
+    case mu_sql_password_plaintext:
       rc = strcmp (sql_pass, pass);
       break;
     }
@@ -663,55 +691,22 @@ mu_sql_authenticate (struct mu_auth_data **return_data MU_ARG_UNUSED,
   return rc == 0 ? 0 : MU_ERR_AUTH_FAILURE;
 }
 
-int
-mu_sql_module_init (enum mu_gocs_op op, void *data)
-{
-  struct mu_sql_module_config *cfg = data;
-
-  if (op != mu_gocs_op_set)
-    return 0;
-  mu_sql_module_config.interface = mu_sql_interface_index (cfg->interface);
-  if (mu_sql_module_config.interface == 0)
-    {
-      mu_error (_("unknown SQL interface `%s'"), cfg->interface);
-      return 1;
-    }
-
-  mu_sql_module_config.getpwnam_query = cfg->getpwnam_query;   
-  mu_sql_module_config.getpass_query  = cfg->getpass_query;    
-  mu_sql_module_config.getpwuid_query = cfg->getpwuid_query;   
-  mu_sql_module_config.host = cfg->host;             
-  mu_sql_module_config.user = cfg->user;             
-  mu_sql_module_config.passwd = cfg->passwd;           
-  mu_sql_module_config.db = cfg->db;               
-  mu_sql_module_config.port = cfg->port;             
-  mu_sql_module_config.password_type = cfg->password_type;    
-  mu_sql_module_config.field_map = cfg->field_map;        
-
-  return 0;
-}
-
 #else
 
 # define mu_sql_authenticate mu_auth_nosupport
 # define mu_auth_sql_by_name mu_auth_nosupport
 # define mu_auth_sql_by_uid mu_auth_nosupport
-
+# define mu_sql_param NULL
 #endif
 
 
 struct mu_auth_module mu_auth_sql_module = {
-  "sql",
-#ifdef USE_SQL
-  mu_sql_module_init,
-#else
-  NULL,
-#endif
-  mu_sql_authenticate,
-  NULL,
-  mu_auth_sql_by_name,
-  NULL,
-  mu_auth_sql_by_uid,
-  NULL
+  .name = "sql",
+  .cfg = mu_sql_param,
+  .handler = {
+    [mu_auth_authenticate] = mu_sql_authenticate,
+    [mu_auth_getpwnam]     = mu_auth_sql_by_name,
+    [mu_auth_getpwuid]     = mu_auth_sql_by_uid
+  }
 };
 
