@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2002, 2005, 2006, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001-2002, 2005-2007, 2010-2012, 2014-2017 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,40 +13,31 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
-typedef struct _alias *alias_t;
-
-struct _alias
-{
-  mu_list_t list;
-};
-
 static mu_assoc_t aliases;
 
 static void
 alias_free (void *data)
 {
-  alias_t al = data;
-  util_slist_destroy (&al->list);
+  mu_list_t al = data;
+  util_slist_destroy (&al);
 }
 
 static void
-alias_print_group (const char *name, alias_t al)
+alias_print_group (const char *name, mu_list_t al)
 {
-  fprintf (ofile, "%s    ", name);
-  util_slist_print (al->list, 0);
-  fprintf (ofile, "\n");
+  mu_printf ("%s    ", name);
+  util_slist_print (al, 0);
+  mu_printf ("\n");
 }
 
-static alias_t
+static mu_list_t
 alias_lookup (const char *name)
 {
-  return mu_assoc_ref (aliases, name);
+  return mu_assoc_get (aliases, name);
 }
 
 static void
@@ -63,7 +55,7 @@ alias_print (char *name)
 	   mu_iterator_next (itr))
 	{
 	  const char *name;
-	  alias_t al;
+	  mu_list_t al;
 	  if (mu_iterator_current_kv (itr, (const void **)&name, (void**)&al))
 	    continue;
 	  alias_print_group (name, al);
@@ -71,12 +63,12 @@ alias_print (char *name)
     }
   else
     {
-      alias_t al;
+      mu_list_t al;
 
       al = alias_lookup (name);
       if (!al)
 	{
-	  util_error (_("\"%s\": not a group"), name);
+	  mu_error (_("\"%s\": not a group"), name);
 	  return;
 	}
       alias_print_group (name, al);
@@ -84,21 +76,25 @@ alias_print (char *name)
 }
 
 static int
-alias_create (const char *name, alias_t *al)
+alias_create (const char *name, mu_list_t *al)
 {
   int rc;
-
+  mu_list_t l;
+  
   if (!aliases)
     {
-      mu_assoc_create (&aliases, sizeof (struct _alias), 0);
-      mu_assoc_set_free (aliases, alias_free);
+      mu_assoc_create (&aliases, 0);
+      mu_assoc_set_destroy_item (aliases, alias_free);
+    }
+  if (mu_assoc_lookup_ref (aliases, name, al))
+    {
+      rc = mu_list_create (&l);
+      if (rc)
+	return rc;
+      mu_assoc_install (aliases, name, l);
+      *al = l;
+      return 0;
     }
-  
-  rc = mu_assoc_ref_install (aliases, name, (void**) al);
-  if (rc == MU_ERR_EXISTS)
-    return 0;
-  if (rc == 0)
-    return mu_list_create (&(*al)->list);
   return 1;
 }
 
@@ -112,7 +108,7 @@ alias_destroy (const char *name)
 static void
 recursive_alias_expand (const char *name, mu_list_t exlist, mu_list_t origlist)
 { 
-  alias_t al;
+  mu_list_t al;
   mu_iterator_t itr;
   
   if ((al = alias_lookup (name)) == NULL)
@@ -122,7 +118,7 @@ recursive_alias_expand (const char *name, mu_list_t exlist, mu_list_t origlist)
       return;
     }
   
-  mu_list_get_iterator (al->list, &itr);
+  mu_list_get_iterator (al, &itr);
   for (mu_iterator_first (itr);
        !mu_iterator_is_done (itr);
        mu_iterator_next (itr))
@@ -132,9 +128,9 @@ recursive_alias_expand (const char *name, mu_list_t exlist, mu_list_t origlist)
       mu_iterator_current (itr, (void **)&word);
       if (mu_list_locate (origlist, word, NULL) == MU_ERR_NOENT)
 	{
-	  mu_list_prepend (origlist, word);
+	  mu_list_push (origlist, word);
 	  recursive_alias_expand (word, exlist, origlist);
-	  mu_list_remove (origlist, word);
+	  mu_list_pop (origlist, NULL);
 	}
     }
   mu_iterator_destroy (&itr);
@@ -149,10 +145,10 @@ string_comp (const void *item, const void *value)
 char *
 alias_expand (const char *name)
 {
-  alias_t al;
+  mu_list_t al;
   mu_list_t list;
   
-  if (util_getenv (NULL, "recursivealiases", Mail_env_boolean, 0) == 0)
+  if (mailvar_get (NULL, "recursivealiases", mailvar_type_boolean, 0) == 0)
     {
       char *s;
       mu_list_t origlist;
@@ -160,14 +156,14 @@ alias_expand (const char *name)
       int status = mu_list_create (&list);
       if (status)
 	{
-	  mu_error (_("Cannot create list: %s"), mu_strerror (status));
+	  mu_diag_funcall (MU_DIAG_ERROR, "mu_list_create", NULL, status);
 	  return NULL;
 	}
       status = mu_list_create (&origlist);
       if (status)
 	{
 	  mu_list_destroy (&origlist);
-	  mu_error (_("Cannot create list: %s"), mu_strerror (status));
+	  mu_diag_funcall (MU_DIAG_ERROR, "mu_list_create", NULL, status);
 	  return NULL;
 	}
       mu_list_set_comparator (list, string_comp);
@@ -181,7 +177,7 @@ alias_expand (const char *name)
   
   if ((al = alias_lookup (name)) == NULL)
     return NULL;
-  return util_slist_to_string (al->list, ",");
+  return util_slist_to_string (al, ",");
 }
 
 
@@ -199,7 +195,7 @@ alias_iterate_next (alias_iterator_t atr)
   while (!mu_iterator_is_done (atr->itr))
     {
       const char *name;
-      alias_t al;
+      mu_list_t al;
 
       if (mu_iterator_current_kv (atr->itr, (const void **)&name, (void**)&al))
 	continue;
@@ -226,7 +222,7 @@ alias_iterate_first (const char *prefix, alias_iterator_t *pc)
   if (mu_assoc_get_iterator (aliases, &itr))
     return NULL;
   mu_iterator_first (itr);
-  atr = xmalloc (sizeof *atr);
+  atr = mu_alloc (sizeof *atr);
   atr->prefix = prefix;
   atr->prefixlen = strlen (prefix);
   atr->pos = 0;
@@ -260,7 +256,7 @@ mail_alias (int argc, char **argv)
     alias_print (argv[1]);
   else
     {
-      alias_t al;
+      mu_list_t al;
 
       if (alias_create (argv[1], &al))
 	return 1;
@@ -268,7 +264,7 @@ mail_alias (int argc, char **argv)
       argc--;
       argv++;
       while (--argc)
-	util_slist_add (&al->list, *++argv);
+	util_slist_add (&al, *++argv);
     }
   return 0;
 }
