@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2005, 
-   2007 Free Software Foundation, Inc.
+   Copyright (C) 1999-2002, 2005, 2007, 2009-2012, 2014-2017 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 #include <sys/stat.h>
@@ -35,14 +33,14 @@ sig_handler (int signo)
   switch (signo)
     {
     case SIGINT:
-      if (util_getenv (NULL, "quit", Mail_env_boolean, 0) == 0)
+      if (mailvar_get (NULL, "quit", mailvar_type_boolean, 0) == 0)
 	exit (0);
       _interrupted++;
       break;
 #if defined (SIGWINCH)
     case SIGWINCH:
-      util_do_command ("set screen=%d", util_getlines());
-      util_do_command ("set columns=%d", util_getcols());
+      util_do_command ("set screen=%d", util_getlines ());
+      util_do_command ("set columns=%d", util_getcols ());
       page_invalidate (1);
       break;
 #endif
@@ -95,7 +93,8 @@ ml_readline_init ()
 
 #ifdef WITH_READLINE
   rl_readline_name = "mail";
-  rl_attempted_completion_function = (CPPFunction*)ml_command_completion;
+  rl_attempted_completion_function =
+    (rl_completion_func_t*) ml_command_completion;
   rl_getc_function = ml_getc;
 #endif
 #ifdef HAVE_SIGACTION
@@ -120,62 +119,29 @@ ml_readline_init ()
 char *
 ml_readline_internal ()
 {
-  char *line;
-  char *p;
-  size_t alloclen, linelen;
+  char *buf = NULL;
+  size_t size = 0, n;
+  int rc;
 
-  p = line = xcalloc (1, 255);
-  alloclen = 255;
-  linelen = 0;
-  for (;;)
+  rc = mu_stream_getline (mu_strin, &buf, &size, &n);
+  if (rc)
     {
-      size_t n;
-
-      p = fgets (p, alloclen - linelen, stdin);
-
-      if (p)
-	n = strlen(p);
-      else if (_interrupted)
-	{
-	  free (line);
-	  return NULL;
-	}
-      else
-	n = 0;
-
-      linelen += n;
-
-      /* Error.  */
-      if (linelen == 0)
-	{
-	  free (line);
-	  return NULL;
-	}
-
-      /* Ok.  */
-      if (line[linelen - 1] == '\n')
-	{
-	  line[linelen - 1] = '\0';
-	  return line;
-	}
-      else
-        {
-	  char *tmp;
-	  alloclen *= 2;
-	  tmp = realloc (line, alloclen);
-	  if (tmp == NULL)
-	    {
-	      free (line);
-	      return NULL;
-	    }
-	  line = tmp;
-	  p = line + linelen;
-	}
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_stream_getline", NULL, rc);
+      return NULL;
     }
+  if (_interrupted)
+    {
+      free (buf);
+      return NULL;
+    }
+  if (n == 0)
+    return NULL;
+  mu_rtrim_cset (buf, "\n");
+  return buf;
 }
 
 char *
-ml_readline (char *prompt)
+ml_readline (const char *prompt)
 {
   if (interactive)
     return readline (prompt);
@@ -183,11 +149,11 @@ ml_readline (char *prompt)
 }
 
 char *
-ml_readline_with_intr (char *prompt)
+ml_readline_with_intr (const char *prompt)
 {
   char *str = ml_readline (prompt);
   if (_interrupted)
-    printf ("\n");
+    mu_printf ("\n");
   return str;
 }
 
@@ -232,32 +198,38 @@ ml_reread (const char *prompt, char **text)
 char **
 ml_command_completion (char *cmd, int start, int end)
 {
-  int argc;
-  char **argv;
   char **ret;
   char *p;
+  struct mu_wordsplit ws;
 
-  for (p = rl_line_buffer; p < rl_line_buffer + start && isspace (*p); p++)
+  for (p = rl_line_buffer; p < rl_line_buffer + start && mu_isblank (*p); p++)
     ;
-  
-  if (mu_argcv_get_n (p, end, NULL, NULL, &argc, &argv))
-    return NULL;
+
+  if (mu_wordsplit_len (p, end, &ws, MU_WRDSF_DEFFLAGS))
+    {
+      mu_error (_("mu_wordsplit_len failed: %s"),
+		mu_wordsplit_strerror (&ws));
+      return NULL;
+    }
   rl_completion_append_character = ' ';
-  
-  if (argc == 0 || (argc == 1 && strlen (argv[0]) <= end - start))
+
+  if (ws.ws_wordc == 0 ||
+      (ws.ws_wordc == 1 && strlen (ws.ws_wordv[0]) <= end - start))
     {
       ret = rl_completion_matches (cmd, ml_command_generator);
       rl_attempted_completion_over = 1;
     }
   else
     {
-      const struct mail_command_entry *entry = mail_find_command (argv[0]);
+      const struct mail_command_entry *entry =
+	mail_find_command (ws.ws_wordv[0]);
       if (entry && entry->command_completion)
-	ret = entry->command_completion (argc, argv, start == end);
+	ret = entry->command_completion (ws.ws_wordc, ws.ws_wordv,
+					 start == end);
       else
 	ret = NULL;
     }
-  mu_argcv_free (argc, argv);
+  mu_wordsplit_free (&ws);
   return ret;
 }
 
@@ -270,7 +242,7 @@ ml_command_generator (const char *text, int state)
   static int i, len;
   const char *name;
   const struct mail_command *cp;
-  
+
   if (!state)
     {
       i = 0;
@@ -284,7 +256,7 @@ ml_command_generator (const char *text, int state)
 	name = cp->shortname;
       i++;
       if (strncmp (name, text, len) == 0)
-	return strdup (name);
+	return mu_strdup (name);
     }
 
   return NULL;
@@ -319,14 +291,6 @@ msglist_compl (int argc, char **argv, int ws)
   return NULL;
 }
 
-char **
-msglist_file_compl (int argc, char **argv, int ws MU_ARG_UNUSED)
-{
-  if (argc == 1)
-    ml_attempted_completion_over ();
-  return NULL;
-}
-
 char **
 command_compl (int argc, char **argv, int ws)
 {
@@ -342,25 +306,25 @@ command_compl (int argc, char **argv, int ws)
 
    Select only those files that match given FLAGS (MU_FOLDER_ATTRIBUTE_*
    constants).
-   
+
    STATE is 0 for the first call, 1 otherwise.
  */
 static char *
 file_generator (const char *text, int state,
 		char *path, size_t pathlen,
-		char repl, 
+		char repl,
 		int flags)
 {
   static mu_list_t list;
   static mu_iterator_t itr;
-  
+
   if (!state)
     {
       char *wcard;
       mu_folder_t folder;
       size_t count;
-      
-      wcard = xmalloc (strlen (text) + 2);
+
+      wcard = mu_alloc (strlen (text) + 2);
       strcat (strcpy (wcard, text), "*");
 
       if (mu_folder_create (&folder, path))
@@ -368,7 +332,7 @@ file_generator (const char *text, int state,
 	  free (wcard);
 	  return NULL;
 	}
-      
+
       mu_folder_list (folder, path, wcard, 1, &list);
       free (wcard);
       mu_folder_destroy (&folder);
@@ -380,7 +344,7 @@ file_generator (const char *text, int state,
 	}
       else if (count == 1)
 	ml_set_completion_append_character (0);
-      
+
       if (mu_list_get_iterator (list, &itr))
 	{
 	  mu_list_destroy (&list);
@@ -401,8 +365,8 @@ file_generator (const char *text, int state,
 	    {
 	      size_t len = strlen (resp->name + pathlen);
 	      char *ptr;
-	      
-	      ret = xmalloc (len + (repl ? 1 : 0) + 1);
+
+	      ret = mu_alloc (len + (repl ? 1 : 0) + 1);
 	      ptr = ret;
 	      if (repl)
 		*ptr++ = repl;
@@ -410,7 +374,7 @@ file_generator (const char *text, int state,
 	      ptr[len] = 0;
 	    }
 	  else
-	    ret = xstrdup (resp->name);
+	    ret = mu_strdup (resp->name);
 	  return ret;
 	}
     }
@@ -424,13 +388,13 @@ folder_generator (const char *text, int state)
 {
   char *ret;
   static size_t pathlen;
-  
+
   if (!state)
     {
       char *path = util_folder_path ("");
       if (!path)
 	return NULL;
-      
+
       pathlen = strlen (path);
       ret = file_generator (text, state, path, pathlen, '+',
 			    MU_FOLDER_ATTRIBUTE_ALL);
@@ -442,8 +406,102 @@ folder_generator (const char *text, int state)
   return ret;
 }
 
-char **
-file_compl (int argc, char **argv, int ws)
+static char *
+msgtype_generator (const char *text, int state)
+{
+  static char types[] = "dnorTtu";
+  static int i;
+  char *s;
+
+  if (!state)
+    {
+      if (text[1])
+	return NULL;
+      i = 0;
+    }
+  if (!types[i])
+    return NULL;
+  s = malloc (2);
+  if (s)
+    {
+      s[0] = types[i++];
+      s[1] = 0;
+    }
+  return s;
+}
+
+static char *
+header_generator (const char *text, int state)
+{
+  static int i, len;
+  char *hdr;
+  char *hdrlist[] = {
+    MU_HEADER_RETURN_PATH,
+    MU_HEADER_RECEIVED,
+    MU_HEADER_DATE,
+    MU_HEADER_DCC,
+    MU_HEADER_FROM,
+    MU_HEADER_SENDER,
+    MU_HEADER_RESENT_FROM,
+    MU_HEADER_SUBJECT,
+    MU_HEADER_RESENT_SENDER,
+    MU_HEADER_TO,
+    MU_HEADER_RESENT_TO,
+    MU_HEADER_CC,
+    MU_HEADER_RESENT_CC,
+    MU_HEADER_BCC,
+    MU_HEADER_RESENT_BCC,
+    MU_HEADER_REPLY_TO,
+    MU_HEADER_RESENT_REPLY_TO,
+    MU_HEADER_MESSAGE_ID,
+    MU_HEADER_RESENT_MESSAGE_ID,
+    MU_HEADER_IN_REPLY_TO,
+    MU_HEADER_REFERENCE,
+    MU_HEADER_REFERENCES,
+    MU_HEADER_ENCRYPTED,
+    MU_HEADER_PRECEDENCE,
+    MU_HEADER_STATUS,
+    MU_HEADER_CONTENT_LENGTH,
+    MU_HEADER_CONTENT_LANGUAGE,
+    MU_HEADER_CONTENT_TRANSFER_ENCODING,
+    MU_HEADER_CONTENT_ID,
+    MU_HEADER_CONTENT_TYPE,
+    MU_HEADER_CONTENT_DESCRIPTION,
+    MU_HEADER_CONTENT_DISPOSITION,
+    MU_HEADER_CONTENT_MD5,
+    MU_HEADER_CONTENT_LOCATION,
+    MU_HEADER_MIME_VERSION,
+    MU_HEADER_X_MAILER,
+    MU_HEADER_X_UIDL,
+    MU_HEADER_X_UID,
+    MU_HEADER_X_IMAPBASE,
+    MU_HEADER_ENV_SENDER,
+    MU_HEADER_ENV_DATE,
+    MU_HEADER_FCC,
+    MU_HEADER_DELIVERY_DATE,
+    MU_HEADER_ENVELOPE_TO,
+    MU_HEADER_X_EXPIRE_TIMESTAMP,
+    NULL
+  };
+
+  if (!state)
+    {
+      i = 0;
+      len = strlen (text);
+    }
+
+  while ((hdr = hdrlist[i]))
+    {
+      i++;
+      if (mu_c_strncasecmp (hdr, text, len) == 0)
+	return mu_strdup (hdr);
+    }
+
+  return NULL;
+}
+
+static char **
+file_compl_internal (int argc, char **argv, int ws, int msglist)
 {
   char *text;
 
@@ -453,26 +511,59 @@ file_compl (int argc, char **argv, int ws)
       ml_attempted_completion_over ();
       return NULL;
     }
-  
+
   text = argv[argc-1];
+
+  if (msglist)
+    {
+      if (text[0] == ':')
+	{
+	  ml_set_completion_append_character (' ');
+	  return rl_completion_matches (text, msgtype_generator);
+	}
+      else if (mu_isalpha (text[0]))
+	{
+	  ml_set_completion_append_character (':');
+	  return rl_completion_matches (text, header_generator);
+	}
+      else if (mu_isdigit (text[0]))
+	{
+	  ml_attempted_completion_over ();
+	  return NULL;
+	}
+    }
+
   switch (text[0])
     {
     case '+':
       text++;
       break;
 
+    case '%':
     case '#':
     case '&':
       ml_attempted_completion_over ();
       return NULL;
-      
+
     default:
       return NULL; /* Will be expanded by readline itself */
     }
-  
+
   return rl_completion_matches (text, folder_generator);
 }
 
+char **
+file_compl (int argc, char **argv, int ws)
+{
+  return file_compl_internal (argc, argv, ws, 0);
+}
+
+char **
+msglist_file_compl (int argc, char **argv, int ws)
+{
+  return file_compl_internal (argc, argv, ws, 1);
+}
+
 static char *
 dir_generator (const char *text, int state)
 {
@@ -504,17 +595,17 @@ dir_generator (const char *text, int state)
 	  /* else FIXME! */
 
 	case '/':
-	  path = strdup (text);
+	  path = mu_strdup (text);
 	  pathlen = 0;
 	  repl = 0;
 	  break;
-	  
+
 	default:
-	  path = strdup ("./");
+	  path = mu_strdup ("./");
 	  pathlen = 2;
 	  repl = 0;
 	}
-      
+
       ret = file_generator (text, state, path, pathlen, repl,
 			    MU_FOLDER_ATTRIBUTE_DIRECTORY);
       free (path);
@@ -542,7 +633,7 @@ alias_generator (const char *text, int state)
 {
   static alias_iterator_t itr;
   const char *p;
-  
+
   if (!state)
     p = alias_iterate_first (text, &itr);
   else
@@ -553,7 +644,7 @@ alias_generator (const char *text, int state)
       alias_iterate_end (&itr);
       return NULL;
     }
-  return strdup (p);
+  return mu_strdup (p);
 }
 
 char **
@@ -563,32 +654,6 @@ alias_compl (int argc, char **argv, int ws)
   return rl_completion_matches (ws ? "" : argv[argc-1], alias_generator);
 }
 
-static char *
-var_generator (const char *text, int state)
-{
-  static var_iterator_t itr;
-  const char *p;
-  
-  if (!state)
-    p = var_iterate_first (text, &itr);
-  else
-    p = var_iterate_next (itr);
-
-  if (!p)
-    {
-      var_iterate_end (&itr);
-      return NULL;
-    }
-  return strdup (p);
-}
-
-char **
-var_compl (int argc, char **argv, int ws)
-{
-  ml_attempted_completion_over ();
-  return rl_completion_matches (ws ? "" : argv[argc-1], var_generator);
-}
-
 static char *
 mkfilename (const char *dir, const char *file)
 {
@@ -612,7 +677,7 @@ exec_generator (const char *text, int state)
   static size_t dsize;
   static DIR *dp;
   struct dirent *ent;
-  
+
   if (!state)
     {
       var = getenv ("PATH");
@@ -635,13 +700,13 @@ exec_generator (const char *text, int state)
 	    break;
 	  else
 	    var++;
-	  
+
 	  p = strchr (var, ':');
 	  if (!p)
 	    len = strlen (var) + 1;
 	  else
 	    len = p - var + 1;
-	  
+
 	  if (dsize == 0)
 	    {
 	      dir = malloc (len);
@@ -652,13 +717,13 @@ exec_generator (const char *text, int state)
 	      dir = realloc (dir, len);
 	      dsize = len;
 	    }
-	  
+
 	  if (!dir)
 	    return NULL;
 	  memcpy (dir, var, len - 1);
 	  dir[len - 1] = 0;
 	  var += len - 1;
-	  
+
 	  dp = opendir (dir);
 	  if (!dp)
 	    continue;
@@ -669,18 +734,18 @@ exec_generator (const char *text, int state)
 	  char *name = mkfilename (dir, ent->d_name);
 	  if (name)
 	    {
-	      int rc = access (name, X_OK); 
+	      int rc = access (name, X_OK);
 	      if (rc == 0)
 		{
 		  struct stat st;
 		  rc = !(stat (name, &st) == 0 && S_ISREG (st.st_mode));
 		}
-		    
+
 	      free (name);
 	      if (rc == 0
 		  && strlen (ent->d_name) >= prefix_len
 		  && strncmp (ent->d_name, text, prefix_len) == 0)
-		return strdup (ent->d_name);
+		return mu_strdup (ent->d_name);
 	    }
 	}
 
@@ -825,7 +890,7 @@ ml_reread (const char *prompt, char **text)
       line = strdup (*text);
       if (line)
 	{
-	  pos = strlen(line);
+	  pos = strlen (line);
 	  line_size = pos + 1;
 	}
     }
@@ -838,7 +903,7 @@ ml_reread (const char *prompt, char **text)
 
   if (!line)
     {
-      util_error (_("Not enough memory to edit the line"));
+      mu_error (_("Not enough memory to edit the line"));
       return -1;
     }
 
@@ -866,7 +931,7 @@ ml_reread (const char *prompt, char **text)
 	  if ((p = realloc (line, line_size + LINE_INC)) == NULL)
 	    {
 	      fputs ("\n", stdout);
-	      util_error (_("Not enough memory to edit the line"));
+	      mu_error (_("Not enough memory to edit the line"));
 	      break;
 	    }
 	  else
@@ -913,7 +978,7 @@ ml_reread (const char *prompt, char **text)
 	      if ((p = realloc (line, line_size + LINE_INC)) == NULL)
 		{
 		  fputs ("\n", stdout);
-		  util_error (_("Not enough memory to edit the line"));
+		  mu_error (_("Not enough memory to edit the line"));
 		  break;
 		}
 	      else
@@ -949,7 +1014,7 @@ ml_reread (const char *prompt, char **text)
 	  if ((p = realloc (line, line_size + LINE_INC)) == NULL)
 	    {
 	      fputs ("\n", stdout);
-	      util_error (_("Not enough memory to edit the line"));
+	      mu_error (_("Not enough memory to edit the line"));
 	      break;
 	    }
 	  else
@@ -981,8 +1046,8 @@ readline (char *prompt)
 {
   if (prompt)
     {
-      fprintf (ofile, "%s", prompt);
-      fflush (ofile);
+      mu_printf ("%s", prompt);
+      mu_stream_flush (mu_strout);
     }
 
   return ml_readline_internal ();
@@ -999,4 +1064,3 @@ ml_attempted_completion_over ()
 }
 
 #endif
-
