@@ -1,5 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2005, 2007 Free Software Foundation, Inc.
+   Copyright (C) 1999, 2001, 2005, 2007, 2010-2012, 2014-2017 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -12,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 #include "mail.h"
 
@@ -227,6 +226,8 @@ static const struct mail_escape_entry mail_escape_table[] = {
   {"!",	"!",	"![shell-command]", escape_shell },
   {":",	":",	":[mail-command]",  escape_command },
   {"-",	"-",	"-[mail-command]",  escape_command },
+  {"+",	"+",	"+name [content-type [encoding]]", escape_attach },
+  {"^",	"^",	"^N",               escape_remove_attachment },
   {"?",	"?",	"?",		    escape_help },
   {"A",	"A",	"A",		    escape_sign },
   {"a",	"a",	"a",		    escape_sign },
@@ -238,6 +239,7 @@ static const struct mail_escape_entry mail_escape_table[] = {
   {"F",	"F",	"F[mesg-list]",	    escape_print },
   {"h",	"h",	"h",		    escape_headers },
   {"i",	"i",	"i[var-name]",	    escape_insert },
+  {"l",	"l",	"l",                escape_list_attachments },
   {"m",	"m",	"m[mesg-list]",	    escape_quote },
   {"M",	"M",	"M[mesg-list]",	    escape_quote },
   {"p",	"p",	"p",		    escape_type_input },
