@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999-2003, 2005-2007, 2010-2012, 2014-2016 Free
+   Copyright (C) 1999-2003, 2005-2007, 2010-2012, 2014-2017 Free
    Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
@@ -178,11 +178,9 @@ has_folder_option (struct mu_option *opt)
 }
 
 void
-mh_getopt0 (int *pargc, char ***pargv, struct mu_option *options,
-	    int mhflags,
-	    char *argdoc, char *progdoc, char *extradoc,
-	    int (*eatarg) (int argc, char **argv, void *data),
-	    void *data)
+mh_getopt (int *pargc, char ***pargv, struct mu_option *options,
+	   int mhflags,
+	   char *argdoc, char *progdoc, char *extradoc)
 {
   int argc = *pargc;
   char **argv = *pargv;
@@ -199,9 +197,6 @@ mh_getopt0 (int *pargc, char ***pargv, struct mu_option *options,
   po.po_negation = "no";
   flags |= MU_PARSEOPT_NEGATION;
 
-  if (eatarg)
-    flags |= MU_PARSEOPT_IN_ORDER;
-  
   if ((mhflags & MH_GETOPT_DEFAULT_FOLDER) || has_folder_option (options))
     {
       po.po_special_args = N_("[+FOLDER]");
@@ -259,29 +254,12 @@ mh_getopt0 (int *pargc, char ***pargv, struct mu_option *options,
   if (options)
     optv[i++] = options;
   optv[i] = NULL;
+  
+  if (mu_parseopt (&po, argc, argv, optv, flags))
+    exit (po.po_exit_error);
 
-  do
-    {
-      if (mu_parseopt (&po, argc, argv, optv, flags))
-	exit (po.po_exit_error);
-      argc -= po.po_arg_start;
-      argv += po.po_arg_start;
-      if (eatarg)
-	{
-	  int consumed = eatarg (argc, argv, data);
-	  if (consumed == 0)
-	    break;
-	  else
-	    {
-	      argc -= consumed;
-	      argv += consumed;
-	    }
-	  flags |= MU_PARSEOPT_ARGV0;
-	}
-      else
-	break;
-    }
-  while (argc > 0);
+  argc -= po.po_arg_start;
+  argv += po.po_arg_start;
 
   process_std_options (argc, argv, &po);
   
@@ -304,15 +282,6 @@ mh_getopt0 (int *pargc, char ***pargv, struct mu_option *options,
   mh_init2 ();
 }
 
-void
-mh_getopt (int *pargc, char ***pargv, struct mu_option *options,
-	   int mhflags,
-	   char *argdoc, char *progdoc, char *extradoc)
-{
-  return mh_getopt0 (pargc, pargv, options, mhflags, argdoc, progdoc, extradoc,
-		     NULL, NULL);
-}
-
 void
 mh_opt_notimpl (struct mu_parseopt *po, struct mu_option *opt, char const *arg)
 {
