@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2006, 2007, 2009,
-   2010 Free Software Foundation, Inc.
+   Copyright (C) 1999-2003, 2005-2007, 2009-2012, 2014-2017 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -13,9 +13,7 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with GNU Mailutils; if not, write to the Free Software
-   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
-   MA 02110-1301 USA */
+   along with GNU Mailutils.  If not, see <http://www.gnu.org/licenses/>. */
 
 /* Initialize MH applications. */
 
@@ -45,11 +43,10 @@ char mh_list_format[] =
 void
 mh_init ()
 {
+  mu_stdstream_setup (MU_STDSTREAM_RESET_NONE);
+  
   /* Register all mailbox and mailer formats */
   mu_register_all_formats ();
-#ifdef WITH_TLS
-  mu_init_tls_libs ();
-#endif
 
   /* Read user's profile */
   mh_read_profile ();
@@ -59,32 +56,45 @@ void
 mh_init2 ()
 {
   mh_current_folder ();
-  mh_global_sequences_get ("cur", NULL);
 }
 
 int
-mh_read_formfile (char *name, char **pformat)
+mh_read_formfile (char const *name, char **pformat)
 {
   FILE *fp;
   struct stat st;
   char *ptr;
   size_t off = 0;
   char *format_str;
-
-  if (stat (name, &st))
+  char *file_name;
+  int rc;
+  
+  rc = mh_find_file (name, &file_name);
+  if (rc)
     {
-      mu_error (_("cannot stat format file %s: %s"), name, strerror (errno));
+      mu_error (_("cannot access format file %s: %s"), name, strerror (rc));
       return -1;
     }
   
-  fp = fopen (name, "r");
-  if (!fp)
+  if (stat (file_name, &st))
     {
-      mu_error (_("cannot open format file %s: %s"), name, strerror (errno));
+      mu_error (_("cannot stat format file %s: %s"), file_name,
+		strerror (errno));
+      free (file_name);
       return -1;
     }
-
-  format_str = xmalloc (st.st_size+1);
+  
+  fp = fopen (file_name, "r");
+  if (!fp)
+    {
+      mu_error (_("cannot open format file %s: %s"), file_name,
+		strerror (errno));
+      free (file_name);
+      return -1;
+    }
+  free (file_name);
+  
+  format_str = mu_alloc (st.st_size+1);
   while ((ptr = fgets (format_str + off, st.st_size - off + 1, fp)) != NULL)
     {
       int len = strlen (ptr);
@@ -137,7 +147,7 @@ mh_get_my_name (char *name)
       name = pw->pw_name;
     }
 
-  my_name = strdup (name);
+  my_name = mu_strdup (name);
   my_email = mu_get_user_email (name);
 }
 
@@ -160,7 +170,7 @@ mh_is_my_name (const char *name)
   char *pname, *p;
   int rc = 0;
   
-  pname = strdup (name);
+  pname = mu_strdup (name);
   p = strchr (pname, '@');
   if (p)
     for (p++; *p; p++)
@@ -200,7 +210,7 @@ mh_is_my_name (const char *name)
 	      while (len > 0 && mu_isspace (p[len-1]))
 		len--;
 
-	      pat = xmalloc (len + 1);
+	      pat = mu_alloc (len + 1);
 	      memcpy (pat, p, len);
 	      pat[len] = 0;
 	      rc = emailcmp (pat, pname) == 0;
@@ -224,7 +234,7 @@ static int
 make_dir_hier (const char *p, mode_t perm)
 {
   int rc = 0;
-  char *dir = xstrdup (p);
+  char *dir = mu_strdup (p);
   char *q = dir;
 
   while (!rc && (q = strchr (q + 1, '/')))
@@ -380,17 +390,10 @@ mh_audit_open (char *name, mu_mailbox_t mbox)
   mu_url_t url;
   char *namep;
   
-  namep = mu_tilde_expansion (name, "/", NULL);
-  if (strchr (namep, '/') == NULL)
+  namep = mu_tilde_expansion (name, MU_HIERARCHY_DELIMITER, NULL);
+  if (strchr (namep, MU_HIERARCHY_DELIMITER) == NULL)
     {
-      char *p = NULL;
-
-      asprintf (&p, "%s/%s", mu_folder_directory (), namep);
-      if (!p)
-	{
-	  mu_error (_("not enough memory"));
-	  exit (1);
-	}
+      char *p = mh_safe_make_file_name (mu_folder_directory (), namep);
       free (namep);
       namep = p;
     }
@@ -429,14 +432,13 @@ mh_message_number (mu_message_t msg, size_t *pnum)
 }
 
 mu_mailbox_t
-mh_open_folder (const char *folder, int create)
+mh_open_folder (const char *folder, int flags)
 {
   mu_mailbox_t mbox = NULL;
   char *name;
-  int flags = MU_STREAM_RDWR;
   
-  name = mh_expand_name (NULL, folder, 1);
-  if (create && mh_check_folder (name, 1))
+  name = mh_expand_name (NULL, folder, NAME_FOLDER);
+  if ((flags & MU_STREAM_CREAT) && mh_check_folder (name, 1))
     exit (0);
     
   if (mu_mailbox_create_default (&mbox, name))
@@ -446,9 +448,6 @@ mh_open_folder (const char *folder, int create)
       exit (1);
     }
 
-  if (create)
-    flags |= MU_STREAM_CREAT;
-  
   if (mu_mailbox_open (mbox, flags))
     {
       mu_error (_("cannot open mailbox %s: %s"), name, strerror (errno));
@@ -469,44 +468,61 @@ mh_get_dir ()
   if (mhdir[0] != '/')
     {
       char *p = mu_get_homedir ();
-      asprintf (&mhcopy, "%s/%s", p, mhdir);
+      mhcopy = mh_safe_make_file_name (p, mhdir);
       free (p);
     }
   else 
     mhcopy = strdup (mhdir);
+  if (!mhcopy)
+    {
+      mu_error (_("not enough memory"));
+      abort ();
+    }
   return mhcopy;
 }
 
 char *
-mh_expand_name (const char *base, const char *name, int is_folder)
+mh_expand_name (const char *base, const char *name, int what)
 {
   char *p = NULL;
   char *namep = NULL;
   
-  namep = mu_tilde_expansion (name, "/", NULL);
+  namep = mu_tilde_expansion (name, MU_HIERARCHY_DELIMITER, NULL);
   if (namep[0] == '+')
     memmove (namep, namep + 1, strlen (namep)); /* copy null byte as well */
   else if (strncmp (namep, "../", 3) == 0 || strncmp (namep, "./", 2) == 0)
     {
       char *cwd = mu_getcwd ();
-      char *tmp = NULL;
-      asprintf (&tmp, "%s/%s", cwd, namep);
+      char *tmp = mh_safe_make_file_name (cwd, namep);
       free (cwd);
+      if (what == NAME_FILE)
+	return tmp;
       free (namep);
       namep = tmp;
     }
   
-  if (is_folder)
+  if (what == NAME_FOLDER)
     {
       if (memcmp (namep, "mh:/", 4) == 0)
 	return namep;
       else if (namep[0] == '/')
-	asprintf (&p, "mh:%s", namep);
+	mu_asprintf (&p, "mh:%s", namep);
       else
-	asprintf (&p, "mh:%s/%s", base ? base : mu_folder_directory (), namep);
+	mu_asprintf (&p, "mh:%s/%s", base ? base : mu_folder_directory (), 
+                     namep);
     }
   else if (namep[0] != '/')
-    asprintf (&p, "%s/%s", base ? base : mu_folder_directory (), namep);
+    {
+        if (what == NAME_FILE)
+	  {
+	    char *cwd = mu_getcwd ();
+	    p = mh_safe_make_file_name (cwd, namep);
+	    free (cwd);
+	  }
+	else
+	  p = mh_safe_make_file_name (base ? base : mu_folder_directory (),
+				      namep);
+    }
   else
     return namep;
   
@@ -515,127 +531,143 @@ mh_expand_name (const char *base, const char *name, int is_folder)
 }
 
 int
-mh_iterate (mu_mailbox_t mbox, mh_msgset_t *msgset,
-	    mh_iterator_fp itr, void *data)
+mh_find_file (const char *name, char **resolved_name)
 {
+  char *s;
   int rc;
-  size_t i;
-
-  for (i = 0; i < msgset->count; i++)
+  
+  if (name[0] == '/' ||
+      (name[0] == '.' && name[1] == '/') ||
+      (name[0] == '.' && name[1] == '.' && name[2] == '/'))
     {
-      mu_message_t msg;
-      size_t num;
-
-      num = msgset->list[i];
-      if ((rc = mu_mailbox_get_message (mbox, num, &msg)) != 0)
-	{
-	  mu_error (_("cannot get message %lu: %s"),
-		    (unsigned long) num, mu_strerror (rc));
-	  return 1;
-	}
-
-      itr (mbox, msg, num, data);
+      *resolved_name = mu_strdup (name);
+      if (access (name, R_OK) == 0)
+	return 0;
+      return errno;
     }
 
-  return 0;
-}
-
-int
-mh_spawnp (const char *prog, const char *file)
-{
-  int argc, i, rc, status;
-  char **argv, **xargv;
-
-  if ((rc = mu_argcv_get (prog, "", "#", &argc, &argv)) != 0)
+  if (name[0] == '~')
     {
-      mu_diag_funcall (MU_DIAG_ERROR, "mu_argcv_get", prog, rc);
-      mu_argcv_free (argc, argv);
-      return 1;
+      s = mu_tilde_expansion (name, MU_HIERARCHY_DELIMITER, NULL);
+      *resolved_name = s;
+      if (access (s, R_OK) == 0)
+	return 0;
+      return errno;
     }
-
-  xargv = calloc (argc + 2, sizeof (*xargv));
-  if (!xargv)
+  
+  s = mh_expand_name (NULL, name, NAME_ANY);
+  if (access (s, R_OK) == 0)
     {
-      mh_err_memory (0);
-      mu_argcv_free (argc, argv);
-      return 1;
+      *resolved_name = s;
+      return 0;
     }
+  if (errno != ENOENT)
+    mu_diag_output (MU_DIAG_WARNING,
+		    _("cannot access %s: %s"), s, mu_strerror (errno));
+  free (s);
 
-  for (i = 0; i < argc; i++)
-    xargv[i] = argv[i];
-  xargv[i++] = (char*) file;
-  xargv[i++] = NULL;
+  s = mh_expand_name (mh_global_profile_get ("mhetcdir", MHLIBDIR), name,
+                      NAME_ANY);
+  if (access (s, R_OK) == 0)
+    {
+      *resolved_name = s;
+      return 0;
+    }
+  if (errno != ENOENT)
+    mu_diag_output (MU_DIAG_WARNING,
+		    _("cannot access %s: %s"), s, mu_strerror (errno));
+  free (s);
 
-  rc = mu_spawnvp (xargv[0], xargv, &status);
-
-  free (xargv);
-  mu_argcv_free (argc, argv);
+  *resolved_name = mu_strdup (name);
+  if (access (name, R_OK) == 0)
+    return 0;
+  rc = errno;
+  if (rc != ENOENT)
+    mu_diag_output (MU_DIAG_WARNING,
+		    _("cannot access %s: %s"), s, mu_strerror (rc));
 
   return rc;
 }
 
 int
-mh_file_copy (const char *from, const char *to)
+mh_spawnp (const char *prog, const char *file)
 {
-  char *buffer;
-  size_t bufsize, rdsize;
-  struct stat st;
-  mu_stream_t in;
-  mu_stream_t out;
-  int rc;
-  
-  if (stat (from, &st))
+  struct mu_wordsplit ws;
+  size_t i;
+  int rc, status;
+  char **xargv;
+
+  ws.ws_comment = "#";
+  if (mu_wordsplit (prog, &ws, MU_WRDSF_DEFFLAGS | MU_WRDSF_COMMENT))
     {
-      mu_error ("mh_copy: %s", mu_strerror (errno));
-      return -1;
+      mu_error (_("cannot split line `%s': %s"), prog,
+		mu_wordsplit_strerror (&ws));
+      return 1;
     }
 
-  for (bufsize = st.st_size; bufsize > 0 && (buffer = malloc (bufsize)) == 0;
-       bufsize /= 2)
-    ;
+  xargv = calloc (ws.ws_wordc + 2, sizeof (*xargv));
+  if (!xargv)
+    {
+      mh_err_memory (0);
+      mu_wordsplit_free (&ws);
+      return 1;
+    }
 
-  if (!bufsize)
-    mh_err_memory (1);
+  for (i = 0; i < ws.ws_wordc; i++)
+    xargv[i] = ws.ws_wordv[i];
+  xargv[i++] = (char*) file;
+  xargv[i++] = NULL;
+  
+  rc = mu_spawnvp (xargv[0], xargv, &status);
 
-  if ((rc = mu_file_stream_create (&in, from, MU_STREAM_READ)) != 0
-      || (rc = mu_stream_open (in)))
+  free (xargv);
+  mu_wordsplit_free (&ws);
+
+  return rc;
+}
+
+/* Copy data from FROM to TO, creating the latter if necessary.
+   FIXME: How about formats?
+*/
+int
+mh_file_copy (const char *from, const char *to)
+{
+  mu_stream_t in, out, flt;
+  int rc;
+  
+  if ((rc = mu_file_stream_create (&in, from, MU_STREAM_READ)))
     {
       mu_error (_("cannot open input file `%s': %s"),
 		from, mu_strerror (rc));
-      free (buffer);
       return 1;
     }
 
-  if ((rc = mu_file_stream_create (&out, to, MU_STREAM_RDWR|MU_STREAM_CREAT)) != 0
-      || (rc = mu_stream_open (out)))
+  if ((rc = mu_file_stream_create (&out, to, MU_STREAM_RDWR|MU_STREAM_CREAT)))
     {
       mu_error (_("cannot open output file `%s': %s"),
 		to, mu_strerror (rc));
-      free (buffer);
-      mu_stream_close (in);
-      mu_stream_destroy (&in, mu_stream_get_owner (in));
+      mu_stream_destroy (&in);
       return 1;
     }
 
-  while (st.st_size > 0
-	 && (rc = mu_stream_sequential_read (in, buffer, bufsize, &rdsize)) == 0
-	 && rdsize > 0)
+  rc = mu_filter_create (&flt, in, "INLINE-COMMENT", MU_FILTER_DECODE,
+			 MU_STREAM_READ);
+  mu_stream_unref (in);
+  if (rc)
     {
-      if ((rc = mu_stream_sequential_write (out, buffer, rdsize)) != 0)
-	{
-	  mu_error (_("write error on `%s': %s"),
-		    to, mu_strerror (rc));
-	  break;
-	}
-      st.st_size -= rdsize;
+      mu_error (_("cannot open filter stream: %s"), mu_strerror (rc));
+      mu_stream_destroy (&out);
+      return 1;
     }
+  
+  rc = mu_stream_copy (out, flt, 0, NULL);
+      
+  mu_stream_destroy (&flt);
+  mu_stream_destroy (&out);
 
-  free (buffer);
-
-  mu_stream_close (in);
-  mu_stream_close (out);
-  mu_stream_destroy (&in, mu_stream_get_owner (in));
-  mu_stream_destroy (&out, mu_stream_get_owner (out));
+  if (rc)
+    mu_error (_("error copying file `%s' to `%s': %s"),
+	      from, to, mu_strerror (rc));
   
   return rc;
 }
@@ -660,14 +692,6 @@ _file_to_message (const char *file_name)
       return NULL;
     }
   
-  if ((rc = mu_stream_open (instream)))
-    {
-      mu_error (_("cannot open input stream (file %s): %s"),
-		file_name, mu_strerror (rc));
-      mu_stream_destroy (&instream, mu_stream_get_owner (instream));
-      return NULL;
-    }
-
   return mh_stream_to_message (instream);
 }
 
@@ -679,7 +703,7 @@ mh_file_to_message (const char *folder, const char *file_name)
   
   if (folder)
     {
-      tmp_name = mh_expand_name (folder, file_name, 0);
+      tmp_name = mh_expand_name (folder, file_name, NAME_ANY);
       msg = _file_to_message (tmp_name);
       free (tmp_name);
     }
@@ -708,14 +732,13 @@ mh_real_install (char *name, int automode)
   char *home = mu_get_homedir ();
   char *mhdir;
   char *ctx;
-  FILE *fp;
-  
-  asprintf (&mhdir, "%s/%s", home, "Mail");
+  int rc;
+  mu_stream_t profile;
+
+  mhdir = mh_safe_make_file_name (home, "Mail");
   
   if (!automode)
     {
-      size_t n = 0;
-      
       /* TRANSLATORS: This is a question and will be followed
 	 by question mark on output. */
       if (mh_getyn_interactive (_("Do you need help")))
@@ -726,29 +749,27 @@ mh_real_install (char *name, int automode)
       if (!mh_getyn_interactive (_("Do you want the standard MH path \"%s\""), mhdir))
 	{
 	  int local;
-	  char *p;
+	  char *p, *buf = NULL;
+	  size_t size = 0;
 	  
 	  /* TRANSLATORS: This is a question and will be followed
 	     by question mark on output. */
 	  local = mh_getyn_interactive (_("Do you want a path below your login directory"));
 	  if (local)
-	    printf (_("What is the path? "));
+	    mu_printf (_("What is the path? "));
 	  else
-	    printf (_("What is the full path? "));
-	  if (getline (&p, &n, stdin) <= 0)
+	    mu_printf (_("What is the full path? "));
+	  mu_stream_flush (mu_strin);
+	  if (mu_stream_getline (mu_strin, &buf, &size, NULL))
 	    exit (1);
-
-	  n = strlen (p);
-	  if (n == 0)
-	    exit (1);
-
-	  if (p[n-1] == '\n')
-	    p[n-1] = 0;
-
+	  p = mu_str_stripws (buf);
+	  if (p > buf)
+	    memmove (buf, p, strlen (p) + 1);
+	  
 	  free (mhdir);
 	  if (local)
 	    {
-	      asprintf (&mhdir, "%s/%s", home, p);
+              mhdir = mh_safe_make_file_name (home, p);
 	      free (p);
 	    }
 	  else
@@ -759,24 +780,26 @@ mh_real_install (char *name, int automode)
   if (mh_check_folder (mhdir, !automode))
     exit (1);
 
-  fp = fopen (name, "w");
-  if (!fp)
+  rc = mu_file_stream_create (&profile, name,
+			      MU_STREAM_WRITE | MU_STREAM_CREAT);
+  if (rc)
     {
-      mu_error (_("cannot open file %s: %s"), name, mu_strerror (errno));
+      mu_error (_("cannot open file %s: %s"), name, mu_strerror (rc));
       exit (1);
     }
-  fprintf (fp, "Path: %s\n", mhdir);
-  fclose (fp);
+  mu_stream_printf (profile, "Path: %s\n", mhdir);
+  mu_stream_destroy (&profile);
 
-  asprintf (&ctx, "%s/%s", mhdir, MH_CONTEXT_FILE);
-  fp = fopen (ctx, "w");
-  if (fp)
+  ctx = mh_safe_make_file_name (mhdir, MH_CONTEXT_FILE);
+  rc = mu_file_stream_create (&profile, ctx,
+			      MU_STREAM_WRITE | MU_STREAM_CREAT);
+  if (rc)
     {
-      fprintf (fp, "Current-Folder: inbox\n");
-      fclose (fp);
+      mu_stream_printf (profile, "Current-Folder: inbox\n");
+      mu_stream_destroy (&profile);
     }
   free (ctx);
-  asprintf (&ctx, "%s/inbox", mhdir);
+  ctx = mh_safe_make_file_name (mhdir, "inbox");
   if (mh_check_folder (ctx, !automode))
     exit (1);
   free (ctx);
@@ -788,7 +811,7 @@ mh_install (char *name, int automode)
 {
   struct stat st;
   
-  if (stat(name, &st))
+  if (stat (name, &st))
     {
       if (errno == ENOENT)
 	{
@@ -805,19 +828,18 @@ mh_install (char *name, int automode)
   else if ((st.st_mode & S_IFREG) || (st.st_mode & S_IFLNK)) 
     {
       mu_error(_("You already have an MH profile, use an editor to modify it"));
-      exit (0);
+      exit (1);
     }
   else
     {
-      mu_error(_("You already have file %s which is not a regular file or a symbolic link.\n"
-		 "Please remove it and try again"),
-	       name);
+      mu_error (_("You already have file %s which is not a regular file or a symbolic link."), name);
+      mu_error (_("Please remove it and try again"));
       exit (1);
     }
 }
         
 void
-mh_annotate (mu_message_t msg, char *field, char *text, int date)
+mh_annotate (mu_message_t msg, const char *field, const char *text, int date)
 {
   mu_header_t hdr;
   mu_attribute_t attr;
@@ -843,14 +865,6 @@ mh_annotate (mu_message_t msg, char *field, char *text, int date)
   mu_attribute_set_modified (attr);
 }
 
-char *
-mh_draft_name ()
-{
-  return mh_expand_name (mh_global_profile_get ("Draft-Folder",
-						mu_folder_directory ()),
-			 "draft", 0);
-}
-
 char *
 mh_create_message_id (int subpart)
 {
@@ -879,25 +893,16 @@ mh_charset (const char *dfl)
     return NULL;
   if (mu_c_strcasecmp (charset, "auto") == 0)
     {
-      /* Try to deduce the charset from LC_ALL variable */
-      
-      char *lc_all = getenv ("LC_ALL");
-      if (lc_all)
+      static char *saved_charset;
+
+      if (!saved_charset)
 	{
-	  char *sp;
-	  char *lang;
-	  char *terr;
-
-	  char *tmp = strdup (lc_all);
-	  lang = strtok_r (tmp, "_", &sp);
-	  terr = strtok_r (NULL, ".", &sp);
-	  charset = strtok_r (NULL, "@", &sp);
-
-	  if (!charset)
-	    charset = mu_charset_lookup (lang, terr);
-	  
-	  free (tmp);
+	  /* Try to deduce the charset from LC_ALL variable */
+	  struct mu_lc_all lc_all;
+	  if (mu_parse_lc_all (getenv ("LC_ALL"), &lc_all, MU_LC_CSET) == 0)
+	    saved_charset = lc_all.charset; /* FIXME: Mild memory leak */
 	}
+      charset = saved_charset;
     }
   return charset;
 }
@@ -925,7 +930,7 @@ mh_quote (const char *in, char **out)
         if (*p == '\\' || *p == '"')
 	  len++;
 
-      *out = xmalloc (len + 1);
+      *out = mu_alloc (len + 1);
       q = *out;
       p = in;
       *q++ = *p++;
@@ -939,7 +944,7 @@ mh_quote (const char *in, char **out)
       *q = 0;
     }
   else
-    *out = xstrdup (in);
+    *out = mu_strdup (in);
 }
 
 void
@@ -987,57 +992,66 @@ mh_draft_message (const char *name, const char *msgspec, char **pname)
   mu_url_t url;
   size_t uid;
   int rc;
-  const char *urlstr;
   mu_mailbox_t mbox;
-
-  mbox = mh_open_folder (name, 0);
+  const char *path;
+  
+  mbox = mh_open_folder (name, MU_STREAM_RDWR);
   if (!mbox)
     return 1;
   
   mu_mailbox_get_url (mbox, &url);
-  urlstr = mu_url_to_string (url);
 
   if (strcmp (msgspec, "new") == 0)
     {
+      mu_property_t prop;
+      
       rc = mu_mailbox_uidnext (mbox, &uid);
       if (rc)
-	mu_error (_("cannot obtain sequence number for the new message: %s"),
-		  mu_strerror (rc));
+	{
+	  mu_error (_("cannot obtain sequence number for the new message: %s"),
+		    mu_strerror (rc));
+	  exit (1);
+	}
+      mu_mailbox_get_property (mbox, &prop);
+      mu_property_set_value (prop, "cur", mu_umaxtostr (0, uid), 1);
     }
   else
     {
       char *argv[2];
-      mh_msgset_t msgset;
+      mu_msgset_t msgset;
       
       argv[0] = (char*) msgspec;
       argv[1] = NULL;
-      rc = mh_msgset_parse (mbox, &msgset, 1, argv, "cur");
-      if (rc)
-	mu_error (_("invalid message number: %s"), msgspec);
-      else if (msgset.count > 1)
+      mh_msgset_parse (&msgset, mbox, 1, argv, "cur");
+      if (!mh_msgset_single_message (msgset))
 	mu_error (_("only one message at a time!"));
       else
-	uid = msgset.list[0];
-
-      mh_msgset_free (&msgset);
+	uid = mh_msgset_first_uid (msgset);
+      mu_msgset_free (msgset);
     }
-  
-  if (rc == 0)
+
+  mu_url_sget_path (url, &path);
+  rc = mu_asprintf (pname, "%s/%lu", path, (unsigned long) uid);
+  if (rc)
     {
-      const char *dir;
-      const char *msg;
-      size_t len;
-      
-      dir = urlstr + 3; /* FIXME */
-      
-      msg = mu_umaxtostr (0, uid);
-      len = strlen (dir) + 1 + strlen (msg) + 1;
-      *pname = xmalloc (len);
-      strcpy (*pname, dir);
-      strcat (*pname, "/");
-      strcat (*pname, msg);
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_asprintf", NULL, rc);
+      exit (1);
     }
   mu_mailbox_close (mbox);
   mu_mailbox_destroy (&mbox);
   return rc;
 }
+
+char *
+mh_safe_make_file_name (const char *dir, const char *file)
+{
+  char *name = mu_make_file_name (dir, file);
+  if (!name)
+    {
+      mu_diag_funcall (MU_DIAG_ERROR, "mu_make_file_name", NULL, ENOMEM);
+      abort ();
+    }
+  return name;
+}
+
+			  
