@@ -1,5 +1,5 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2010-2012, 2014-2016 Free Software Foundation, Inc.
+   Copyright (C) 2010-2012, 2014-2017 Free Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -69,39 +69,19 @@ smtp_session_str (enum smtp_session_status stat)
 }
 
 static void
-smtp_prompt_env ()
+smtp_prompt_env (void)
 {
+  mu_assoc_t assoc = mutool_shell_prompt_assoc ();
   const char *value;
   
-  if (!mutool_prompt_env)
-    mutool_prompt_env = mu_calloc (2*7 + 1, sizeof(mutool_prompt_env[0]));
-
-  mutool_prompt_env[0] = "user";
-  mutool_prompt_env[1] = "[nouser]";
   if (smtp_session_status == smtp_session_logged_in &&
       mu_smtp_get_param (smtp, MU_SMTP_PARAM_USERNAME, &value) == 0)
-    mutool_prompt_env[1] = (char*) value;
+    mu_assoc_install (assoc, "user", (void*) value);
 
-  mutool_prompt_env[2] = "host"; 
-  mutool_prompt_env[3] = (smtp_session_status != smtp_session_disconnected) ?
-                           host : "[nohost]";
-
-  mutool_prompt_env[4] = "program-name";
-  mutool_prompt_env[5] = (char*) mu_program_name;
-
-  mutool_prompt_env[6] = "canonical-program-name";
-  mutool_prompt_env[7] = "mu";
-
-  mutool_prompt_env[8] = "package";
-  mutool_prompt_env[9] = PACKAGE;
-
-  mutool_prompt_env[10] = "version";
-  mutool_prompt_env[11] = PACKAGE_VERSION;
-
-  mutool_prompt_env[12] = "status";
-  mutool_prompt_env[13] = (char*) smtp_session_str (smtp_session_status);
-
-  mutool_prompt_env[14] = NULL;
+  if (smtp_session_status != smtp_session_disconnected)
+    mu_assoc_install (assoc, "host", host);
+  mu_assoc_install (assoc, "status",
+		    (void*) smtp_session_str (smtp_session_status));
 }
 
 static void
@@ -258,7 +238,7 @@ com_connect (int argc, char **argv)
   else
     {
       connect_argc = argc;
-      connect_argv = mu_calloc (argc, sizeof (*connect_argv));
+      connect_argv = mu_calloc (argc + 1, sizeof (*connect_argv));
       for (i = 0; i < argc; i++)
 	connect_argv[i] = mu_strdup (argv[i]);
       connect_argv[i] = NULL;
