@@ -1,6 +1,6 @@
 /* GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 1999, 2001, 2003, 2007, 2009-2012 Free Software
-   Foundation, Inc.
+   Copyright (C) 1999, 2001, 2003, 2007, 2009-2012, 2014-2017 Free
+   Software Foundation, Inc.
 
    GNU Mailutils is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -137,6 +137,7 @@ pop3d_session_init (struct pop3d_session *session)
   pop3d_append_capa_string (session, "UIDL", NULL);
   pop3d_append_capa_string (session, "RESP-CODES", NULL);
   pop3d_append_capa_string (session, "PIPELINING", NULL);
+  pop3d_append_capa_string (session, "AUTH-RESP-CODE", NULL);
   if (pop3d_xlines)
     pop3d_append_capa_string (session, "XLINES", NULL);
 
