@@ -1,6 +1,5 @@
-/*
-   GNU Mailutils -- a suite of utilities for electronic mail
-   Copyright (C) 2009, 2010 Free Software Foundation, Inc.
+/* GNU Mailutils -- a suite of utilities for electronic mail
+   Copyright (C) 2009-2012, 2014-2017 Free Software Foundation, Inc.
 
    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
@@ -13,14 +12,12 @@
    Lesser General Public License for more details.
 
    You should have received a copy of the GNU Lesser General
-   Public License along with this library; if not, write to the
-   Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-   Boston, MA 02110-1301 USA
-*/
+   Public License along with this library.  If not, see 
+   <http://www.gnu.org/licenses/>. */
 
 #include "libmu_py.h"
 
-inline PyObject *
+PyObject *
 _ro (PyObject *obj)
 {
   Py_INCREF (obj);
@@ -45,14 +42,23 @@ status_object (int status, PyObject *py_obj)
 static PyObject *package;
 static PyObject *all;
 
+#define MU_MODULE_ROOT PY_PACKAGE_NAME "." PY_ROOT_NAME "."
+
+static char *
+_mu_py_module_name (const char *nm)
+{
+  char *buf = malloc (sizeof (MU_MODULE_ROOT) + strlen (nm));
+  if (!buf)
+    abort ();
+  return strcat (strcpy (buf, MU_MODULE_ROOT), nm);
+}
+  
 PyObject *
 _mu_py_attach_module (const char *name, PyMethodDef *methods)
 {
-  PyObject *module, *m;
-
-  char ns[64] = PY_PACKAGE_NAME "." PY_ROOT_NAME ".";
-  strcat (ns, name);
-
+  PyObject *module;
+  char *ns = _mu_py_module_name (name);
+  
   if (!(module = PyImport_AddModule (ns)))
     return NULL;
 
@@ -61,11 +67,12 @@ _mu_py_attach_module (const char *name, PyMethodDef *methods)
 
   Py_INCREF (module);
 
-  if (!(m = Py_InitModule (ns, methods)))
+  if (methods && !Py_InitModule (ns, methods))
     return NULL;
 
   PyList_Append (all, PyString_FromString (name));
-  return m;
+  free (ns);
+  return module;
 }
 
 void
@@ -75,7 +82,6 @@ mu_py_init (void)
   mu_py_init_attribute ();
   mu_py_init_auth ();
   mu_py_init_body ();
-  mu_py_init_debug ();
   mu_py_init_envelope ();
   mu_py_init_header ();
   mu_py_init_folder ();
@@ -114,8 +120,8 @@ mu_py_attach_modules (void)
   _mu_py_attach_attribute ();
   _mu_py_attach_auth ();
   _mu_py_attach_body ();
-  _mu_py_attach_debug ();
   _mu_py_attach_envelope ();
+  _mu_py_attach_errno ();
   _mu_py_attach_header ();
   _mu_py_attach_filter ();
   _mu_py_attach_folder ();
